import type { FormKitFrameworkContext, FormKitTypeDefinition } from '@formkit/core';
import type { FormKitInputs } from '@formkit/inputs';
import { createSection, label, outer } from '@formkit/inputs';
import type { DatePickerEmitsOptions, DatePickerProps } from 'primevue/datepicker';
import PrimevueDatepicker from 'primevue/datepicker';
import { defineComponent, markRaw } from 'vue';
import { floatLabel } from '../sections/floatLabel';
import { messages } from '../sections/messages';
import { help } from '../sections/help';

// :plugins="[castNumber]"
type PickerValue = Date | Array<Date> | Array<Date | null> | undefined | null;
type PropValueToDate = (value: unknown) => PickerValue;
type PropDateToValue = (date?: PickerValue) => unknown;

type PrimevueDatepickerListeners = {
  'onUpdate:modelValue': DatePickerEmitsOptions['update:modelValue'];
  'onBlur': DatePickerEmitsOptions['blur'];
};

const SchemaComponent = defineComponent(
  (vueProps: { context: FormKitFrameworkContext & { valueToDate: PropValueToDate; dateToValue: PropDateToValue } }) => {
    const formkitContext = vueProps.context;
    const { valueToDate, dateToValue } = formkitContext;
    const listeners: PrimevueDatepickerListeners = {
      'onUpdate:modelValue': (value) => {
        let newValue = value as unknown;
        try {
          newValue = dateToValue(value);
        } catch (e) {
          console.error(e);
        }
        formkitContext.node.input(newValue);
      },
      'onBlur': async (e) => {
        setTimeout(
          () => formkitContext.handlers.blur.call(undefined, e as never),
          166, // 因为会触发两次。所以让blur事件延迟一点，可以考虑优化。
        );
      },
    };

    return () => {
      let value = formkitContext._value;
      try {
        value = valueToDate(formkitContext._value);
      } catch (e) {
        console.error(e);
      }
      return (
        <PrimevueDatepicker
          inputId={formkitContext.id}
          fluid
          invalid={formkitContext.state.invalid}
          disabled={!!formkitContext.disabled}
          modelValue={value}
          dateFormat={formkitContext.dateFormat as never}
          manualInput={formkitContext.manualInput as never}
          {...listeners}
        />
      );
    };
  },
  {
    props: ['context'],
  },
);

const input = createSection('input', () => ({
  $cmp: markRaw(SchemaComponent) as never,
  bind: '$attrs',
  props: {
    context: '$node.context',
  },
}));

export const PDatePicker: FormKitTypeDefinition = {
  type: 'input',
  schema: outer(
    floatLabel(
      input(), //
      label('$label'),
    ),
    help('$help'),
    messages(),
  ),
  props: ['valueToDate', 'dateToValue', 'dateFormat'],
  schemaMemoKey: 'q2dkascustq', // Math.random().toString(36).substring(2, 15)
};

declare module '@formkit/inputs' {
  // https://formkit.com/essentials/custom-inputs#typescript-support
  interface FormKitInputProps<Props extends FormKitInputs<Props>> {
    PDatePicker: {
      type: 'PDatePicker';
      valueToDate?: PropValueToDate;
      dateToValue?: PropDateToValue;
      value?: unknown;
      view?: DatePickerProps['view'];
      /**
       * https://primevue.org/datepicker/#format
       */
      dateFormat?: DatePickerProps['dateFormat'];
      manualInput?: DatePickerProps['manualInput'];
    };
  }
}
