import type { FormKitFrameworkContext, FormKitTypeDefinition } from '@formkit/core';
import type { FormKitInputs } from '@formkit/inputs';
import { createSection, label, outer } from '@formkit/inputs';
import PrimevueSelect, { type SelectEmitsOptions, type SelectSlots } from 'primevue/select';
import { computed, defineComponent, markRaw, ref } from 'vue';
import { floatLabel } from '../sections/floatLabel';
import { messages } from '../sections/messages';
import { help } from '../sections/help';

// https://formkit.com/inputs/dropdown

type PrimevueSelectListeners = {
  // 'onFocus': SelectEmitsOptions['focus'];
  'onUpdate:modelValue': SelectEmitsOptions['update:modelValue'];
  // 'onChange': SelectEmitsOptions['change'];
  'onBlur': SelectEmitsOptions['blur'];
};

const SchemaComponent = defineComponent(
  (vueProps: { context: FormKitFrameworkContext }) => {
    const formkitContext = vueProps.context;
    const primevueSelectInstance = ref<{ label: string } | undefined>();
    const listeners: PrimevueSelectListeners = {
      'onUpdate:modelValue': (value: unknown) => {
        formkitContext.node.input(value);
      },
      'onBlur': async (e) => {
        setTimeout(
          () => formkitContext.handlers.blur.call(undefined, e as never),
          166, // 因为会触发两次。所以让blur事件延迟一点，可以考虑优化。
        );
      },
    };
    // [optionsLoader](https://github.com/formkit/formkit/blob/2d5387ba98597775cb2a752af65aee84bc438863/packages/inputs/src/features/options.ts#L125)

    const p_options = ref();
    const loading = ref(true);

    const loadOptions = async () => {
      try {
        let result;
        if (formkitContext.options instanceof Promise) {
          result = await formkitContext.options;
        } else if (typeof formkitContext.options === 'function') {
          const funcResult = await (formkitContext.options as () => unknown).call(undefined);
          result = funcResult instanceof Promise ? await funcResult : funcResult;
        } else {
          result = formkitContext.options;
        }
        p_options.value = result;
      } catch (error) {
        console.error('Failed to load options:', error);
        p_options.value = [];
      } finally {
        loading.value = false;
      }
    };

    loadOptions(); // 立即加载options

    const valueSlot = computed(() => {
      if (primevueSelectInstance.value?.label === 'p-emptylabel' && !!formkitContext._value) {
        return ((slotProps) => {
          return slotProps.value as never;
        }) satisfies SelectSlots['value'];
      } else {
        return undefined;
      }
    });

    return () => {
      return (
        <PrimevueSelect
          style={{
            'border-color':
              !formkitContext.state.invalid && valueSlot.value !== undefined ? 'var(--p-yellow-200)' : undefined,
          }}
          ref={primevueSelectInstance}
          labelId={formkitContext.id}
          fluid
          invalid={formkitContext.state.invalid}
          disabled={!!formkitContext.disabled}
          loading={loading.value}
          options={p_options.value}
          modelValue={formkitContext._value}
          optionLabel={formkitContext.optionLabel as never}
          optionValue={formkitContext.optionValue as never}
          {...listeners}
        >
          {{
            value: valueSlot.value,
          }}
        </PrimevueSelect>
      );
    };
  },
  // https://cn.vuejs.org/api/general#definecomponent
  // 目前仍然需要手动声明运行时的 props
  // 在将来，我们计划提供一个 Babel 插件，自动推断并注入运行时 props (就像在单文件组件中的 defineProps 一样)，以便省略运行时 props 的声明。
  {
    props: ['context'],
  },
);

const input = createSection('input', () => ({
  $cmp: markRaw(SchemaComponent) as never,
  bind: '$attrs',
  props: {
    context: '$node.context',
  },
}));

export const PSelect: FormKitTypeDefinition = {
  type: 'input',
  schema: outer(
    floatLabel(
      input(), //
      label('$label'),
    ),
    help('$help'),
    messages(),
  ),
  props: ['options', 'optionLabel', 'optionValue'],
  schemaMemoKey: 'nip9j70lb1', // Math.random().toString(36).substring(2, 15)
};

// eslint-disable-next-line @typescript-eslint/no-explicit-any
type OptionsItem = Record<string, any>;
type OptionsType = Array<OptionsItem>;

declare module '@formkit/inputs' {
  // https://formkit.com/essentials/custom-inputs#typescript-support
  interface FormKitInputProps<Props extends FormKitInputs<Props>> {
    PSelect: {
      type: 'PSelect';
      options: OptionsType | Promise<OptionsType> | (() => OptionsType | Promise<OptionsType>);
    };
  }
}
